@@ -11,15 +11,15 @@ import unittest
 
 from pathlib import Path
 sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
-from init_paths import init_test_paths
+from init_paths import init_test_paths  # noqa: E402
 init_test_paths(False)
 
-from PySide6.QtBluetooth import QBluetoothLocalDevice
+from PySide6.QtBluetooth import QBluetoothLocalDevice  # noqa: E402
 
 
 class QBluetoothLocalDeviceTest(unittest.TestCase):
     def testInitialization(self):
-        device = QBluetoothLocalDevice()
+        device = QBluetoothLocalDevice()  # noqa: F841
 
 
 if __name__ == '__main__':
