@@ -4,6 +4,7 @@
 #include "typesystem.h"
 #include "arraytypeentry.h"
 #include "codesnip.h"
+#include "cpptypeentry.h"
 #include "complextypeentry.h"
 #include "configurabletypeentry.h"
 #include "constantvaluetypeentry.h"
@@ -33,10 +34,10 @@
 
 #include "qtcompat.h"
 
-#include <QtCore/QDebug>
-#include <QtCore/QRegularExpression>
-#include <QtCore/QSet>
-#include <QtCore/QVarLengthArray>
+#include <QtCore/qdebug.h>
+#include <QtCore/qregularexpression.h>
+#include <QtCore/qset.h>
+#include <QtCore/qvarlengtharray.h>
 
 using namespace Qt::StringLiterals;
 
@@ -75,7 +76,6 @@ public:
     QVersionNumber m_version;
     SourceLocation m_sourceLocation; // XML file
     TypeEntry::CodeGeneration m_codeGeneration = TypeEntry::GenerateCode;
-    TypeEntryPtr m_viewOn;
     CustomTypeEntryPtr m_targetLangApiType;
     int m_revision = 0;
     int m_sbkIndex = 0;
@@ -544,16 +544,6 @@ bool TypeEntry::isComplex() const
     return false;
 }
 
-TypeEntryPtr TypeEntry::viewOn() const
-{
-    return m_d->m_viewOn;
-}
-
-void TypeEntry::setViewOn(const TypeEntryPtr &v)
-{
-    m_d->m_viewOn = v;
-}
-
 TypeEntry *TypeEntry::clone() const
 {
     return new TypeEntry(new TypeEntryPrivate(*m_d.data()));
@@ -573,23 +563,187 @@ void TypeEntry::useAsTypedef(const TypeEntryCPtr &source)
     m_d->m_version = source->m_d->m_version;
 }
 
-// ----------------- CustomTypeEntry
-class  CustomTypeEntryPrivate : public TypeEntryPrivate
+// ----------------- CppTypeEntry
+
+class CppTypeEntryPrivate : public TypeEntryPrivate
 {
 public:
     using TypeEntryPrivate::TypeEntryPrivate;
 
+    CppTypeEntryCPtr m_viewOn;
+    QString m_defaultConstructor;
+    TypeSystem::CopyableFlag m_copyableFlag = TypeSystem::CopyableFlag::Unspecified;
+    bool m_copyableDetected = true;
+    TypeSystem::DefaultConstructibleFlag m_defaultConstructibleFlag = TypeSystem::DefaultConstructibleFlag::Unspecified;
+    bool m_defaultConstructibleDetected = true;
+    TypeSystem::MovableFlag m_movableFlag = TypeSystem::MovableFlag::Unspecified;
+    bool m_movableDetected = true;
+    TypeSystem::QtMetaTypeRegistration m_qtMetaTypeRegistration =
+        TypeSystem::QtMetaTypeRegistration::Unspecified;
+};
+
+CppTypeEntry::CppTypeEntry(const QString &entryName, Type t,
+                           const QVersionNumber &vr,
+                           const TypeEntryCPtr &parent) :
+    TypeEntry(new CppTypeEntryPrivate(entryName, t, vr, parent))
+{
+}
+
+CppTypeEntry::CppTypeEntry(CppTypeEntryPrivate *d) :
+    TypeEntry(d)
+{
+}
+
+const QString &CppTypeEntry::defaultConstructor() const
+{
+    S_D(const CppTypeEntry);
+    return d->m_defaultConstructor;
+}
+
+void CppTypeEntry::setDefaultConstructor(const QString& defaultConstructor)
+{
+    S_D(CppTypeEntry);
+    d->m_defaultConstructor = defaultConstructor;
+}
+
+CppTypeEntryCPtr CppTypeEntry::viewOn() const
+{
+    S_D(const CppTypeEntry);
+    return d->m_viewOn;
+}
+
+void CppTypeEntry::setViewOn(const CppTypeEntryCPtr &v)
+{
+    S_D(CppTypeEntry);
+    d->m_viewOn = v;
+}
+
+TypeSystem::DefaultConstructibleFlag CppTypeEntry::defaultConstructibleFlag() const
+{
+    S_D(const CppTypeEntry);
+    return d->m_defaultConstructibleFlag;
+}
+
+void CppTypeEntry::setDefaultConstructibleFlag(TypeSystem::DefaultConstructibleFlag flag)
+{
+    S_D(CppTypeEntry);
+    d->m_defaultConstructibleFlag = flag;
+}
+
+bool CppTypeEntry::isDefaultConstructible() const
+{
+    S_D(const CppTypeEntry);
+    switch (d->m_defaultConstructibleFlag) {
+    case TypeSystem::DefaultConstructibleFlag::Enabled:
+        return true;
+    case TypeSystem::DefaultConstructibleFlag::Disabled:
+         return false;
+    case TypeSystem::DefaultConstructibleFlag::Unspecified:
+        break;
+    }
+    return d->m_defaultConstructibleDetected || !d->m_defaultConstructor.isEmpty();
+}
+
+void CppTypeEntry::setDefaultConstructibleDetected(bool c)
+{
+    S_D(CppTypeEntry);
+    d->m_defaultConstructibleDetected = c;
+}
+
+TypeSystem::CopyableFlag CppTypeEntry::copyableFlag() const
+{
+    S_D(const CppTypeEntry);
+    return d->m_copyableFlag;
+}
+
+void CppTypeEntry::setCopyableFlag(TypeSystem::CopyableFlag flag)
+{
+    S_D(CppTypeEntry);
+    d->m_copyableFlag = flag;
+}
+
+bool CppTypeEntry::isCopyable() const
+{
+    S_D(const CppTypeEntry);
+    switch (d->m_copyableFlag) {
+    case TypeSystem::CopyableFlag::Enabled:
+        return true;
+    case TypeSystem::CopyableFlag::Disabled:
+        return false;
+    case TypeSystem::CopyableFlag::Unspecified:
+        break;
+    }
+    return d->m_copyableDetected;
+}
+
+void CppTypeEntry::setCopyableDetected(bool c)
+{
+    S_D(CppTypeEntry);
+    d->m_copyableDetected = c;
+}
+
+// Movable has no detection logic in the code model (yet). It is turned off for namespaces
+TypeSystem::MovableFlag CppTypeEntry::movableFlag() const
+{
+    S_D(const CppTypeEntry);
+    return d->m_movableFlag;
+}
+
+void CppTypeEntry::setMovableFlag(TypeSystem::MovableFlag flag)
+{
+    S_D(CppTypeEntry);
+    d->m_movableFlag = flag;
+}
+
+bool CppTypeEntry::isMovable() const
+{
+    S_D(const CppTypeEntry);
+    switch (d->m_movableFlag) {
+    case TypeSystem::MovableFlag::Enabled:
+        return true;
+    case TypeSystem::MovableFlag::Disabled:
+        return false;
+    case TypeSystem::MovableFlag::Unspecified:
+        break;
+    }
+    return d->m_movableDetected;
+}
+
+TypeSystem::QtMetaTypeRegistration CppTypeEntry::qtMetaTypeRegistration() const
+{
+    S_D(const CppTypeEntry);
+    return d->m_qtMetaTypeRegistration;
+}
+
+void CppTypeEntry::setQtMetaTypeRegistration(TypeSystem::QtMetaTypeRegistration r)
+{
+    S_D(CppTypeEntry);
+    d->m_qtMetaTypeRegistration = r;
+}
+
+TypeEntry *CppTypeEntry::clone() const
+{
+    S_D(const CppTypeEntry);
+    return new CppTypeEntry(new CppTypeEntryPrivate(*d));
+}
+
+// ----------------- CustomTypeEntry
+class CustomTypeEntryPrivate : public CppTypeEntryPrivate
+{
+public:
+    using CppTypeEntryPrivate::CppTypeEntryPrivate;
+
     QString m_checkFunction;
 };
 
 CustomTypeEntry::CustomTypeEntry(const QString &entryName, const QVersionNumber &vr,
                                  const TypeEntryCPtr &parent) :
-    TypeEntry(new CustomTypeEntryPrivate(entryName, CustomType, vr, parent))
+    CppTypeEntry(new CustomTypeEntryPrivate(entryName, CustomType, vr, parent))
 {
 }
 
-CustomTypeEntry::CustomTypeEntry(TypeEntryPrivate *d) :
-    TypeEntry(d)
+CustomTypeEntry::CustomTypeEntry(CustomTypeEntryPrivate *d) :
+    CppTypeEntry(d)
 {
 }
 
@@ -653,7 +807,7 @@ TypeSystem::CPythonType PythonTypeEntry::cPythonType() const
     return d->m_cPythonType;
 }
 
-PythonTypeEntry::PythonTypeEntry(TypeEntryPrivate *d) :
+PythonTypeEntry::PythonTypeEntry(CustomTypeEntryPrivate *d) :
     CustomTypeEntry(d)
 {
 }
@@ -788,18 +942,19 @@ void TypeSystemTypeEntry::setSnakeCase(TypeSystem::SnakeCase sc)
 
 // ----------------- VoidTypeEntry
 VoidTypeEntry::VoidTypeEntry() :
-    TypeEntry(u"void"_s, VoidType, QVersionNumber(0, 0), nullptr)
+    CppTypeEntry(u"void"_s, VoidType, QVersionNumber(0, 0), nullptr)
 {
 }
 
-VoidTypeEntry::VoidTypeEntry(TypeEntryPrivate *d) :
-    TypeEntry(d)
+VoidTypeEntry::VoidTypeEntry(CppTypeEntryPrivate *d) :
+    CppTypeEntry(d)
 {
 }
 
 TypeEntry *VoidTypeEntry::clone() const
 {
-    return new VoidTypeEntry(new TypeEntryPrivate(*d_func()));
+    S_D(const CppTypeEntry);
+    return new VoidTypeEntry(new CppTypeEntryPrivate(*d));
 }
 
 VarargsTypeEntry::VarargsTypeEntry() :
@@ -857,12 +1012,12 @@ TemplateArgumentEntry::TemplateArgumentEntry(TemplateArgumentEntryPrivate *d) :
 }
 
 // ----------------- ArrayTypeEntry
-class ArrayTypeEntryPrivate : public TypeEntryPrivate
+class ArrayTypeEntryPrivate : public CppTypeEntryPrivate
 {
 public:
     explicit ArrayTypeEntryPrivate(const TypeEntryCPtr &nested_type, const QVersionNumber &vr,
                                    const TypeEntryCPtr &parent) :
-        TypeEntryPrivate(u"Array"_s, TypeEntry::ArrayType, vr, parent),
+        CppTypeEntryPrivate(u"Array"_s, TypeEntry::ArrayType, vr, parent),
         m_nestedType(nested_type)
     {
     }
@@ -872,7 +1027,7 @@ public:
 
 ArrayTypeEntry::ArrayTypeEntry(const TypeEntryCPtr &nested_type, const QVersionNumber &vr,
                                const TypeEntryCPtr &parent) :
-    TypeEntry(new ArrayTypeEntryPrivate(nested_type, vr, parent))
+    CppTypeEntry(new ArrayTypeEntryPrivate(nested_type, vr, parent))
 {
     Q_ASSERT(nested_type);
 }
@@ -902,22 +1057,21 @@ TypeEntry *ArrayTypeEntry::clone() const
 }
 
 ArrayTypeEntry::ArrayTypeEntry(ArrayTypeEntryPrivate *d) :
-    TypeEntry(d)
+    CppTypeEntry(d)
 {
 }
 
 // ----------------- PrimitiveTypeEntry
-class PrimitiveTypeEntryPrivate : public TypeEntryPrivate
+class PrimitiveTypeEntryPrivate : public CppTypeEntryPrivate
 {
 public:
     PrimitiveTypeEntryPrivate(const QString &entryName, const QVersionNumber &vr,
                               const TypeEntryCPtr &parent) :
-        TypeEntryPrivate(entryName, TypeEntry::PrimitiveType, vr, parent),
+        CppTypeEntryPrivate(entryName, TypeEntry::PrimitiveType, vr, parent),
         m_preferredTargetLangType(true)
     {
     }
 
-    QString m_defaultConstructor;
     CustomConversionPtr m_customConversion;
     PrimitiveTypeEntryPtr m_referencedTypeEntry;
     uint m_preferredTargetLangType : 1;
@@ -925,28 +1079,10 @@ public:
 
 PrimitiveTypeEntry::PrimitiveTypeEntry(const QString &entryName, const QVersionNumber &vr,
                                        const TypeEntryCPtr &parent) :
-    TypeEntry(new PrimitiveTypeEntryPrivate(entryName, vr, parent))
+    CppTypeEntry(new PrimitiveTypeEntryPrivate(entryName, vr, parent))
 {
 }
 
-QString PrimitiveTypeEntry::defaultConstructor() const
-{
-    S_D(const PrimitiveTypeEntry);
-    return d->m_defaultConstructor;
-}
-
-void PrimitiveTypeEntry::setDefaultConstructor(const QString &defaultConstructor)
-{
-    S_D(PrimitiveTypeEntry);
-    d->m_defaultConstructor = defaultConstructor;
-}
-
-bool PrimitiveTypeEntry::hasDefaultConstructor() const
-{
-    S_D(const PrimitiveTypeEntry);
-    return !d->m_defaultConstructor.isEmpty();
-}
-
 PrimitiveTypeEntryPtr PrimitiveTypeEntry::referencedTypeEntry() const
 {
     S_D(const PrimitiveTypeEntry);
@@ -1026,16 +1162,16 @@ TypeEntry *PrimitiveTypeEntry::clone() const
 }
 
 PrimitiveTypeEntry::PrimitiveTypeEntry(PrimitiveTypeEntryPrivate *d)
-    : TypeEntry(d)
+    : CppTypeEntry(d)
 {
 }
 
 // ----------------- ConfigurableTypeEntry
 
-class ConfigurableTypeEntryPrivate : public TypeEntryPrivate
+class ConfigurableTypeEntryPrivate : public CppTypeEntryPrivate
 {
 public:
-    using TypeEntryPrivate::TypeEntryPrivate;
+    using CppTypeEntryPrivate::CppTypeEntryPrivate;
 
     QString m_configCondition;
 };
@@ -1043,12 +1179,12 @@ public:
 ConfigurableTypeEntry::ConfigurableTypeEntry(const QString &entryName, Type t,
                                              const QVersionNumber &vr,
                                              const TypeEntryCPtr &parent) :
-    TypeEntry(new ConfigurableTypeEntryPrivate(entryName, t, vr, parent))
+    CppTypeEntry(new ConfigurableTypeEntryPrivate(entryName, t, vr, parent))
 {
 }
 
 ConfigurableTypeEntry::ConfigurableTypeEntry(ConfigurableTypeEntryPrivate *d) :
-    TypeEntry(d)
+    CppTypeEntry(d)
 {
 }
 
@@ -1089,7 +1225,9 @@ public:
     FlagsTypeEntryPtr m_flags;
     QString m_cppType;
     QString m_docFile;
+    std::weak_ptr<const EnumTypeEntry> m_aliasTypeEntry;
     TypeSystem::PythonEnumType m_pythonEnumType = TypeSystem::PythonEnumType::Unspecified;
+    EnumTypeEntry::AliasMode m_aliasMode = EnumTypeEntry::AliasMode::NoAlias;
 };
 
 EnumTypeEntry::EnumTypeEntry(const QString &entryName,
@@ -1194,6 +1332,30 @@ void EnumTypeEntry::setDocFile(const QString &df)
     d->m_docFile = df;
 }
 
+EnumTypeEntry::AliasMode EnumTypeEntry::aliasMode() const
+{
+    S_D(const EnumTypeEntry);
+    return d->m_aliasMode;
+}
+
+void EnumTypeEntry::setAliasMode(AliasMode am)
+{
+    S_D(EnumTypeEntry);
+    d->m_aliasMode = am;
+}
+
+EnumTypeEntryCPtr EnumTypeEntry::aliasTypeEntry() const
+{
+    S_D(const EnumTypeEntry);
+    return d->m_aliasTypeEntry.lock();
+}
+
+void EnumTypeEntry::setAliasTypeEntry(const EnumTypeEntryCPtr &entry)
+{
+    S_D(EnumTypeEntry);
+    d->m_aliasTypeEntry = entry;
+}
+
 TypeEntry *EnumTypeEntry::clone() const
 {
     S_D(const EnumTypeEntry);
@@ -1256,10 +1418,10 @@ EnumValueTypeEntry::EnumValueTypeEntry(EnumValueTypeEntryPrivate *d) :
 }
 
 // ----------------- FlagsTypeEntry
-class FlagsTypeEntryPrivate : public TypeEntryPrivate
+class FlagsTypeEntryPrivate : public CppTypeEntryPrivate
 {
 public:
-    using TypeEntryPrivate::TypeEntryPrivate;
+    using CppTypeEntryPrivate::CppTypeEntryPrivate;
 
     QString m_originalName;
     QString m_flagsName;
@@ -1268,7 +1430,7 @@ public:
 
 FlagsTypeEntry::FlagsTypeEntry(const QString &entryName, const QVersionNumber &vr,
                                const TypeEntryCPtr &parent) :
-    TypeEntry(new FlagsTypeEntryPrivate(entryName, FlagsType, vr, parent))
+    CppTypeEntry(new FlagsTypeEntryPrivate(entryName, FlagsType, vr, parent))
 {
 }
 
@@ -1281,7 +1443,7 @@ QString FlagsTypeEntry::buildTargetLangName() const
 }
 
 FlagsTypeEntry::FlagsTypeEntry(FlagsTypeEntryPrivate *d) :
-    TypeEntry(d)
+    CppTypeEntry(d)
 {
 }
 
@@ -1364,7 +1526,6 @@ public:
     FieldModificationList m_fieldMods;
     QList<TypeSystemProperty> m_properties;
     QList<TypeSystemPyMethodDefEntry> m_PyMethodDefEntrys;
-    QString m_defaultConstructor;
     QString m_defaultSuperclass;
     QString m_qualifiedCppName;
     QString m_docFile;
@@ -1377,7 +1538,6 @@ public:
     QString m_polymorphicNameFunction;
     QString m_targetType;
     ComplexTypeEntry::TypeFlags m_typeFlags;
-    ComplexTypeEntry::CopyableFlag m_copyableFlag = ComplexTypeEntry::Unknown;
     QString m_hashFunction;
 
     ComplexTypeEntryCPtr m_baseContainerType;
@@ -1389,8 +1549,6 @@ public:
     TypeSystem::BoolCast m_isNullMode = TypeSystem::BoolCast::Unspecified;
     TypeSystem::QtMetaTypeRegistration m_qtMetaTypeRegistration =
         TypeSystem::QtMetaTypeRegistration::Unspecified;
-    // Determined by AbstractMetaBuilder from the code model.
-    bool m_isValueTypeWithCopyConstructorOnly = false;
 };
 
 ComplexTypeEntry::ComplexTypeEntry(const QString &entryName, TypeEntry::Type t,
@@ -1686,30 +1844,6 @@ void ComplexTypeEntry::setDeleteInMainThread(bool dmt)
     d->m_deleteInMainThread = dmt;
 }
 
-ComplexTypeEntry::CopyableFlag ComplexTypeEntry::copyable() const
-{
-    S_D(const ComplexTypeEntry);
-    return d->m_copyableFlag;
-}
-
-void ComplexTypeEntry::setCopyable(ComplexTypeEntry::CopyableFlag flag)
-{
-    S_D(ComplexTypeEntry);
-    d->m_copyableFlag = flag;
-}
-
-TypeSystem::QtMetaTypeRegistration ComplexTypeEntry::qtMetaTypeRegistration() const
-{
-    S_D(const ComplexTypeEntry);
-    return d->m_qtMetaTypeRegistration;
-}
-
-void ComplexTypeEntry::setQtMetaTypeRegistration(TypeSystem::QtMetaTypeRegistration r)
-{
-    S_D(ComplexTypeEntry);
-    d->m_qtMetaTypeRegistration = r;
-}
-
 QString ComplexTypeEntry::hashFunction() const
 {
     S_D(const ComplexTypeEntry);
@@ -1758,24 +1892,6 @@ void ComplexTypeEntry::setAllowThread(TypeSystem::AllowThread allowThread)
     d->m_allowThread = allowThread;
 }
 
-void ComplexTypeEntry::setDefaultConstructor(const QString& defaultConstructor)
-{
-    S_D(ComplexTypeEntry);
-    d->m_defaultConstructor = defaultConstructor;
-}
-
-QString ComplexTypeEntry::defaultConstructor() const
-{
-    S_D(const ComplexTypeEntry);
-    return d->m_defaultConstructor;
-}
-
-bool ComplexTypeEntry::hasDefaultConstructor() const
-{
-    S_D(const ComplexTypeEntry);
-    return !d->m_defaultConstructor.isEmpty();
-}
-
 TypeSystem::SnakeCase ComplexTypeEntry::snakeCase() const
 {
     S_D(const ComplexTypeEntry);
@@ -1788,18 +1904,6 @@ void ComplexTypeEntry::setSnakeCase(TypeSystem::SnakeCase sc)
     d->m_snakeCase = sc;
 }
 
-bool ComplexTypeEntry::isValueTypeWithCopyConstructorOnly() const
-{
-    S_D(const ComplexTypeEntry);
-    return d->m_isValueTypeWithCopyConstructorOnly;
-}
-
-void ComplexTypeEntry::setValueTypeWithCopyConstructorOnly(bool v)
-{
-    S_D(ComplexTypeEntry);
-    d->m_isValueTypeWithCopyConstructorOnly = v;
-}
-
 QString ComplexTypeEntry::docFile() const
 {
     S_D(const ComplexTypeEntry);
@@ -2076,6 +2180,7 @@ public:
     QString m_resetMethod;
     SmartPointerTypeEntry::Instantiations m_instantiations;
     TypeEntryCList m_excludedInstantiations;
+    CustomConversionPtr m_customConversion;
     TypeSystem::SmartPointerType m_smartPointerType;
 };
 
@@ -2097,6 +2202,11 @@ SmartPointerTypeEntry::SmartPointerTypeEntry(const QString &entryName,
     ComplexTypeEntry(new SmartPointerTypeEntryPrivate(entryName, getterName, smartPointerType,
                                                       refCountMethodName, vr, parent))
 {
+    S_D(SmartPointerTypeEntry);
+    // Pre-set detection values which is not done by the code model since
+    // the clang parser might not see all system library includes.
+    d->m_defaultConstructibleDetected = smartPointerType != TypeSystem::SmartPointerType::Handle;
+    d->m_copyableDetected = smartPointerType != TypeSystem::SmartPointerType::Unique;
 }
 
 TypeSystem::SmartPointerType SmartPointerTypeEntry::smartPointerType() const
@@ -2223,6 +2333,24 @@ QString SmartPointerTypeEntry::getTargetName(const AbstractMetaType &metaType) c
     return fixSmartPointerName(name);
 }
 
+bool SmartPointerTypeEntry::hasCustomConversion() const
+{
+    S_D(const SmartPointerTypeEntry);
+    return bool(d->m_customConversion);
+}
+
+void SmartPointerTypeEntry::setCustomConversion(const CustomConversionPtr &customConversion)
+{
+    S_D(SmartPointerTypeEntry);
+    d->m_customConversion = customConversion;
+}
+
+CustomConversionPtr SmartPointerTypeEntry::customConversion() const
+{
+    S_D(const SmartPointerTypeEntry);
+    return d->m_customConversion;
+}
+
 // ----------------- NamespaceTypeEntry
 class NamespaceTypeEntryPrivate : public ComplexTypeEntryPrivate
 {
@@ -2241,6 +2369,8 @@ NamespaceTypeEntry::NamespaceTypeEntry(const QString &entryName, const QVersionN
                                        const TypeEntryCPtr &parent) :
     ComplexTypeEntry(new NamespaceTypeEntryPrivate(entryName, NamespaceType, vr, parent))
 {
+    S_D(NamespaceTypeEntry);
+    d->m_copyableDetected = d->m_defaultConstructibleDetected = d->m_movableDetected = false;
 }
 
 TypeEntry *NamespaceTypeEntry::clone() const
@@ -2485,6 +2615,8 @@ ObjectTypeEntry::ObjectTypeEntry(const QString &entryName, const QVersionNumber
                                  const TypeEntryCPtr &parent)
     : ComplexTypeEntry(entryName, ObjectType, vr, parent)
 {
+    S_D(ComplexTypeEntry);
+    d->m_defaultConstructibleDetected = d->m_copyableDetected = false;
 }
 
 TypeEntry *ObjectTypeEntry::clone() const
@@ -2526,8 +2658,6 @@ void TypeEntry::formatDebug(QDebug &debug) const
     FORMAT_NONEMPTY_STRING("package", m_d->m_targetLangPackage)
     FORMAT_BOOL("stream", m_d->m_stream)
     FORMAT_BOOL("built-in", m_d->m_builtin)
-    if (m_d->m_viewOn)
-       debug << ", views=" << m_d->m_viewOn->name();
     if (!m_d->m_version.isNull() && m_d->m_version > QVersionNumber(0, 0))
         debug << ", version=" << m_d->m_version;
     if (m_d->m_revision)
@@ -2541,6 +2671,19 @@ void TypeEntry::formatDebug(QDebug &debug) const
     formatList(debug, "extraIncludes", m_d->m_extraIncludes, ", ");
 }
 
+void CppTypeEntry::formatDebug(QDebug &debug) const
+{
+    S_D(const CppTypeEntry);
+    if (d->m_viewOn)
+        debug << ", views=" << d->m_viewOn->name();
+    if (isDefaultConstructible())
+        debug << ", [default constructible]";
+    if (isCopyable())
+        debug << ", [copyable]";
+    if (isMovable())
+        debug << ", [movable]";
+}
+
 void PrimitiveTypeEntry::formatDebug(QDebug &debug) const
 {
     TypeEntry::formatDebug(debug);
@@ -2561,8 +2704,7 @@ void ComplexTypeEntry::formatDebug(QDebug &debug) const
     FORMAT_BOOL("deleteInMainThread", d->m_deleteInMainThread)
     if (d->m_typeFlags != 0)
         debug << ", typeFlags=" << d->m_typeFlags;
-    debug << ", copyableFlag=" << d->m_copyableFlag
-        << ", except=" << int(d->m_exceptionHandling)
+    debug << ", except=" << int(d->m_exceptionHandling)
         << ", snakeCase=" << int(d->m_snakeCase);
     FORMAT_NONEMPTY_STRING("defaultSuperclass", d->m_defaultSuperclass)
     FORMAT_NONEMPTY_STRING("polymorphicIdValue", d->m_polymorphicIdValue)
