@@ -18,6 +18,8 @@
 #include <QtCore/qregularexpression.h>
 #include <QtCore/qxmlstream.h>
 
+#include <cstring>
+
 using namespace Qt::StringLiterals;
 
 QDebug operator<<(QDebug debug, const QtXmlToSphinxImage &i)
@@ -198,9 +200,9 @@ enum class WebXmlTag {
     heading, brief, para, italic, bold, see_also, snippet, dots, codeline,
     table, header, row, item, argument, teletype, link, inlineimage, image,
     list, term, raw, underline, superscript, code, badcode, legalese,
-    rst, section, quotefile,
+    rst, section, quotefile, target, keyword, page, group,
     // ignored tags
-    generatedlist, tableofcontents, quotefromfile, skipto, target, page, group,
+    generatedlist, tableofcontents, quotefromfile, skipto,
     // useless tags
     description, definition, printuntil, relation,
     // Doxygen tags
@@ -251,6 +253,7 @@ static const WebXmlTagHash &webXmlTagHash()
         {u"quotefromfile", WebXmlTag::quotefromfile},
         {u"skipto", WebXmlTag::skipto},
         {u"target", WebXmlTag::target},
+        {u"keyword", WebXmlTag::keyword},
         {u"page", WebXmlTag::page},
         {u"group", WebXmlTag::group},
         {u"description", WebXmlTag::description},
@@ -401,6 +404,7 @@ void QtXmlToSphinx::callHandler(WebXmlTag t, QXmlStreamReader &r)
         handleIgnoredTag(r);
         break;
     case WebXmlTag::target:
+    case WebXmlTag::keyword:
         handleTargetTag(r);
         break;
     case WebXmlTag::page:
@@ -688,15 +692,13 @@ QString QtXmlToSphinx::readSnippet(const QString &location, const QString &ident
 void QtXmlToSphinx::handleHeadingTag(QXmlStreamReader& reader)
 {
     static int headingSize = 0;
-    static char type;
-    static char types[] = { '-', '^' };
+    static char type{};
+    static constexpr const char types[] = R"(#*=-^")";
     QXmlStreamReader::TokenType token = reader.tokenType();
     if (token == QXmlStreamReader::StartElement) {
-        uint typeIdx = reader.attributes().value(u"level"_s).toUInt();
-        if (typeIdx >= sizeof(types))
-            type = types[sizeof(types)-1];
-        else
-            type = types[typeIdx];
+        // Levels are 1..n. We start at #2 since <page> already uses '#' (1) for the title.
+        const auto typeIdx = std::size_t(reader.attributes().value(u"level"_s).toUInt()); // level 1..n
+        type = types[std::min(typeIdx, std::strlen(types) - 1)];
     } else if (token == QXmlStreamReader::EndElement) {
         m_output << disableIndent << Pad(type, headingSize) << "\n\n"
             << enableIndent;
@@ -1305,7 +1307,7 @@ void QtXmlToSphinx::handlePageTag(QXmlStreamReader &reader)
        ? writeEscapedRstText(m_output, title)
        : writeEscapedRstText(m_output, fullTitle);
 
-    m_output << '\n' << Pad('*', size) << "\n\n"
+    m_output << '\n' << Pad('#', size) << "\n\n"
         << enableIndent;
 }
 
@@ -1315,7 +1317,7 @@ void QtXmlToSphinx::handleTargetTag(QXmlStreamReader &reader)
         return;
     const auto  name = reader.attributes().value("name");
     if (!name.isEmpty())
-        m_output << rstLabel(name.toString());
+        m_output << disableIndent << rstLabel(name.toString()) << enableIndent;
 }
 
 void QtXmlToSphinx::handleIgnoredTag(QXmlStreamReader&)
