@@ -18,6 +18,7 @@
 
 #include "basewrapper.h"
 #include "autodecref.h"
+#include "sbkpep.h"
 #include "sbkstring.h"
 #include "sbkstaticstrings.h"
 #include "sbkstaticstrings_p.h"
@@ -26,6 +27,7 @@
 #include <structmember.h>
 
 #include <algorithm>
+#include <cstring>
 
 using namespace Shiboken;
 
@@ -68,7 +70,7 @@ PyObject *GetClassOrModOf(PyObject *ob)
         return _get_class_of_descr(ob);
     if (Py_TYPE(ob) == &PyWrapperDescr_Type)
         return _get_class_of_descr(ob);
-    Py_FatalError("unexpected type in GetClassOrModOf");
+    Py_FatalError("libshiboken: unexpected type in GetClassOrModOf");
     return nullptr;
 }
 
@@ -89,7 +91,7 @@ PyObject *GetTypeKey(PyObject *ob)
     }
     AutoDecRef class_name(PyObject_GetAttr(ob, PyMagicName::qualname()));
     if (class_name.isNull()) {
-        Py_FatalError("Signature: missing class name in GetTypeKey");
+        Py_FatalError("libshiboken: missing class name in GetTypeKey");
         return nullptr;
     }
     return Py_BuildValue("(OO)", module_name.object(), class_name.object());
@@ -173,8 +175,8 @@ PyObject *GetSignature_Function(PyObject *obfunc, PyObject *modifier)
     if (props == nullptr)
         Py_RETURN_NONE;
 
-    int flags = PyCFunction_GET_FLAGS(obfunc);
-    PyObject *func_kind;
+    int flags = PyCFunction_GetFlags(obfunc);
+    PyObject *func_kind{};
     if (PyModule_Check(obtype_mod.object()))
         func_kind = PyName::function();
     else if (flags & METH_CLASS)
@@ -258,7 +260,7 @@ PyObject *get_signature_intern(PyObject *ob, PyObject *modifier)
 
 static PyObject *get_signature(PyObject * /* self */, PyObject *args)
 {
-    PyObject *ob;
+    PyObject *ob{};
     PyObject *modifier = nullptr;
 
     if (!PyArg_ParseTuple(args, "O|O", &ob, &modifier))
@@ -302,32 +304,35 @@ static PyObject *feature_import(PyObject * /* self */, PyObject *args, PyObject
         return ret;
     // feature_import did not handle it, so call the normal import.
     Py_DECREF(ret);
-    static PyObject *builtins = PyEval_GetBuiltins();
-    PyObject *origImportFunc = PyDict_GetItemString(builtins, "__orig_import__");
+    Shiboken::AutoDecRef builtins(PepEval_GetFrameBuiltins());
+    PyObject *origImportFunc = PyDict_GetItemString(builtins.object(), "__orig_import__");
     if (origImportFunc == nullptr) {
-        Py_FatalError("builtins has no \"__orig_import__\" function");
+        Py_FatalError("libshiboken: builtins has no \"__orig_import__\" function");
     }
-    // PYSIDE-3054: Instead of just calling the original import, we temporarily
-    //              reset the whole import function to the previous version.
-    //              This prevents unforeseen recursions like in settrace.
-    PyObject *featureImportFunc = PyDict_GetItemString(builtins, "__import__");
-    Py_INCREF(origImportFunc);
-    Py_INCREF(featureImportFunc);
-    PyDict_SetItemString(builtins, "__import__", origImportFunc);
     ret = PyObject_Call(origImportFunc, args, kwds);
     if (ret) {
+        // PYSIDE-3054: Instead of just calling the original import, we temporarily
+        //              reset the whole import function to the previous version.
+        //              This prevents unforeseen recursions like in settrace.
+        PyObject *featureImportFunc = PyDict_GetItemString(builtins.object(), "__import__");
+        Py_INCREF(origImportFunc);
+        Py_INCREF(featureImportFunc);
+        PyDict_SetItemString(builtins.object(), "__import__", origImportFunc);
+
         // PYSIDE-2029: Intercept after the import to search for PySide usage.
         PyObject *post = PyObject_CallFunctionObjArgs(pyside_globals->feature_imported_func,
                                                       ret, nullptr);
         Py_XDECREF(post);
+
+        PyDict_SetItemString(builtins.object(), "__import__", featureImportFunc);
+        Py_DECREF(origImportFunc);
+        Py_DECREF(featureImportFunc);
+
         if (post == nullptr) {
             Py_DECREF(ret);
             ret = nullptr;
         }
     }
-    PyDict_SetItemString(builtins, "__import__", featureImportFunc);
-    Py_DECREF(origImportFunc);
-    Py_DECREF(featureImportFunc);
     return ret;
 }
 
@@ -535,7 +540,7 @@ static int _finishSignaturesCommon(PyObject *module)
     // the shiboken module (or a test module).
     [[maybe_unused]] const char *name = PyModule_GetName(module);
     if (pyside_globals->finish_import_func == nullptr) {
-        assert(strncmp(name, "PySide6.", 8) != 0);
+        assert(std::strncmp(name, "PySide6.", 8) != 0);
         return 0;
     }
     // Call a Python function which has to finish something as well.
@@ -685,8 +690,8 @@ static PyObject *adjustFuncName(const char *func_name)
     static PyObject *ns = PyModule_GetDict(mapping);
 
     char _path[200 + 1] = {};
-    const char *_name = strrchr(func_name, '.');
-    strncat(_path, func_name, _name - func_name);
+    const char *_name = std::strrchr(func_name, '.');
+    std::strncat(_path, func_name, _name - func_name);
     ++_name;
 
     // This is a very cheap call into `mapping.py`.
@@ -731,14 +736,14 @@ static PyObject *adjustFuncName(const char *func_name)
     // Finally, generate the correct path expression.
     char _buf[250 + 1] = {};
     if (prop_name) {
-        auto _prop_name = String::toCString(prop_name);
+        const auto *_prop_name = String::toCString(prop_name);
         if (is_class_prop)
             snprintf(_buf, sizeof(_buf), "%s.__dict__['%s'].fset", _path, _prop_name);
         else
             snprintf(_buf, sizeof(_buf), "%s.%s.fset", _path, _prop_name);
     }
     else {
-        auto _name = String::toCString(name);
+        const auto *_name = String::toCString(name);
         snprintf(_buf, sizeof(_buf), "%s.%s", _path, _name);
     }
     return String::fromCString(_buf);
@@ -768,7 +773,7 @@ void SetError_Argument(PyObject *args, const char *func_name, PyObject *info)
     AutoDecRef new_func_name(adjustFuncName(func_name));
     if (new_func_name.isNull()) {
         PyErr_Print();
-        Py_FatalError("seterror_argument failed to call update_mapping");
+        Py_FatalError("libshiboken: seterror_argument failed to call update_mapping");
     }
     if (info == nullptr)
         info = Py_None;
@@ -776,13 +781,13 @@ void SetError_Argument(PyObject *args, const char *func_name, PyObject *info)
                                                 args, new_func_name.object(), info, nullptr));
     if (res.isNull()) {
         PyErr_Print();
-        Py_FatalError("seterror_argument did not receive a result");
+        Py_FatalError("libshiboken: seterror_argument did not receive a result");
     }
     PyObject *err{};
     PyObject *msg{};
     if (!PyArg_UnpackTuple(res, func_name, 2, 2, &err, &msg)) {
         PyErr_Print();
-        Py_FatalError("unexpected failure in seterror_argument");
+        Py_FatalError("libshiboken: unexpected failure in seterror_argument");
     }
     PyErr_SetObject(err, msg);
 }
