@@ -266,6 +266,7 @@ type_map.update({
     "QSet": typing.Set,
     "QString": str,
     "QLatin1String": str,
+    "QAnyStringView": str,
     "QStringView": str,
     "QStringList": StringList,
     "quint16": int,
@@ -514,6 +515,7 @@ def init_PySide6_QtCore():
         "PySide6.QtCore.QUrl.ComponentFormattingOptions":
             PySide6.QtCore.QUrl.ComponentFormattingOption,  # mismatch option/enum, why???
         "PyUnicode": typing.Text,
+        "QByteArray": typing.Union[PySide6.QtCore.QByteArray, bytes, bytearray, memoryview],
         "QByteArrayView": PySide6.QtCore.QByteArray,
         "Q_NULLPTR": None,
         "QCalendar.Unspecified": PySide6.QtCore.QCalendar.Unspecified,
@@ -541,6 +543,8 @@ def init_PySide6_QtCore():
         "QVariant.Type": type,  # not so sure here...
         "QVariantMap": typing.Dict[str, Variant],
         "std.chrono.seconds{5}" : ellipsis,
+        "Internal.defaultTryTimeout": 5000,
+        "static_cast<int>(Internal.defaultTryTimeout.count())": 5000
     })
     from shibokensupport.signature.parser import using_snake_case
     if using_snake_case():
@@ -556,6 +560,9 @@ def init_PySide6_QtCore():
     type_map_tuple.update({("PySide6.QtCore.QObject.disconnect", "char*"): str})
     type_map_tuple.update({("PySide6.QtCore.QObject.receivers", "char*"): str})
     type_map_tuple.update({("PySide6.QtCore.qtTrId", "char*"): str})
+    # special case - char default is 'int'.
+    # Here we manually set it to map to 'str'.
+    type_map_tuple.update({("PySide6.QtCore.QLocale.toString", "char"): str})
 
     return locals()
 
@@ -620,7 +627,7 @@ def init_PySide6_QtWidgets():
 def init_PySide6_QtSql():
     from PySide6.QtSql import QSqlDatabase
     type_map.update({
-        "QLatin1StringView(QSqlDatabase.defaultConnection)": QSqlDatabase.defaultConnection,
+        "QSqlDatabase.defaultConnectionName()": "",
         "QVariant.Invalid": Invalid("Variant"),  # not sure what I should create, here...
     })
     return locals()
@@ -736,6 +743,9 @@ def init_testbinding():
 
 # Functions which should return Optional(result) but don't.
 missing_optional_return = {
+    "PySide6.QtCore.QObject.parent",
+    "PySide6.QtGui.QGuiApplication.modalWindow",
+    "PySide6.QtGui.QGuiApplication.screenAt",
     "PySide6.QtWidgets.QApplication.activeModalWidget",
     "PySide6.QtWidgets.QApplication.activePopupWidget",
     "PySide6.QtWidgets.QApplication.activeWindow",
@@ -743,16 +753,41 @@ missing_optional_return = {
     "PySide6.QtWidgets.QApplication.setStyle",
     "PySide6.QtWidgets.QApplication.topLevelAt",
     "PySide6.QtWidgets.QApplication.widgetAt",
+    "PySide6.QtWidgets.QBoxLayout.itemAt",
+    "PySide6.QtWidgets.QBoxLayout.takeAt",
+    "PySide6.QtWidgets.QButtonGroup.checkedButton",
     "PySide6.QtWidgets.QComboBox.completer",
     "PySide6.QtWidgets.QComboBox.lineEdit",
     "PySide6.QtWidgets.QComboBox.validator",
+    "PySide6.QtWidgets.QCompleter.popup",
+    "PySide6.QtWidgets.QFormLayout.itemAt",
+    "PySide6.QtWidgets.QFormLayout.takeAt",
+    "PySide6.QtWidgets.QGraphicsAnchorLayout.itemAt",
+    "PySide6.QtWidgets.QGraphicsGridLayout.itemAt",
+    "PySide6.QtWidgets.QGraphicsLayout.itemAt",
+    "PySide6.QtWidgets.QGraphicsLinearLayout.itemAt",
+    "PySide6.QtWidgets.QGraphicsScene.itemAt",
+    "PySide6.QtWidgets.QGraphicsView.itemAt",
     "PySide6.QtWidgets.QGridLayout.itemAt",
     "PySide6.QtWidgets.QGridLayout.itemAtPosition",
+    "PySide6.QtWidgets.QGridLayout.takeAt",
     "PySide6.QtWidgets.QLayout.itemAt",
+    "PySide6.QtWidgets.QLayout.replaceWidget",
+    "PySide6.QtWidgets.QLayout.takeAt",
+    "PySide6.QtWidgets.QListWidget.itemAt",
+    "PySide6.QtWidgets.QScrollArea.widget",
+    "PySide6.QtWidgets.QSplitter.widget",
+    "PySide6.QtWidgets.QStackedLayout.itemAt",
+    "PySide6.QtWidgets.QStackedLayout.takeAt",
+    "PySide6.QtWidgets.QStackedLayout.widget",
+    "PySide6.QtWidgets.QStackedWidget.widget",
+    "PySide6.QtWidgets.QTabWidget.widget",
     "PySide6.QtWidgets.QTableWidget.horizontalHeaderItem",
     "PySide6.QtWidgets.QTableWidget.item",
     "PySide6.QtWidgets.QTableWidget.itemAt",
     "PySide6.QtWidgets.QTableWidget.mimeData",
+    "PySide6.QtWidgets.QToolBox.widget",
+    "PySide6.QtWidgets.QTreeWidget.itemAt",
     "PySide6.QtWidgets.QTreeWidget.takeTopLevelItem",
     "PySide6.QtWidgets.QTreeWidget.topLevelItem",
     "PySide6.QtWidgets.QWidget.childAt",
